append_to_inittab

Macro append_to_inittab 

macro_rules! append_to_inittab {
    ($module:ident) => { ... };
}
Available on crate feature tls and neither PyPy nor GraalPy only.
Add the module to the initialization table in order to make embedded Python code to use it. Module name is the argument.

Use it before Python::initialize and leave feature auto-initialize off